@@ -1,6 +1,6 @@
 # Scalar::Util.pm
 #
-# Copyright (c) 1997-2001 Graham Barr <gbarr@pobox.com>. All rights reserved.
+# Copyright (c) 1997-2005 Graham Barr <gbarr@pobox.com>. All rights reserved.
 # This program is free software; you can redistribute it and/or
 # modify it under the same terms as Perl itself.
 
@@ -10,8 +10,9 @@ require Exporter;
 require List::Util; # List::Util loads the XS
 
 @ISA       = qw(Exporter);
-@EXPORT_OK = qw(blessed dualvar reftype weaken isweak tainted readonly openhandle refaddr isvstring looks_like_number);
-$VERSION   = $VERSION = $List::Util::VERSION;
+@EXPORT_OK = qw(blessed dualvar reftype weaken isweak tainted readonly openhandle refaddr isvstring looks_like_number set_prototype);
+$VERSION    = "1.16";
+$VERSION   = eval $VERSION;
 
 sub export_fail {
   if (grep { /^(weaken|isweak)$/ } @_ ) {
@@ -22,9 +23,9 @@ sub export_fail {
     require Carp;
     Carp::croak("Vstrings are not implemented in the version of perl");
   }
-  if (grep { /^dualvar$/ } @_ ) {
+  if (grep { /^(dualvar|set_prototype)$/ } @_ ) {
     require Carp;
-    Carp::croak("dualvar is only avaliable with the XS version");
+    Carp::croak("$1 is only avaliable with the XS version");
   }
 
   @_;
@@ -50,7 +51,7 @@ sub openhandle ($) {
 
 eval <<'ESQ' unless defined &dualvar;
 
-push @EXPORT_FAIL, qw(weaken isweak dualvar isvstring);
+push @EXPORT_FAIL, qw(weaken isweak dualvar isvstring set_prototype);
 
 # The code beyond here is only used if the XS is not installed
 
@@ -67,7 +68,8 @@ sub blessed ($) {
 sub refaddr($) {
   my $pkg = ref($_[0]) or return undef;
   bless $_[0], 'Scalar::Util::Fake';
-  my $i = int($_[0]);
+  "$_[0]" =~ /0x(\w+)/;
+  my $i = do { local $^W; hex $1 };
   bless $_[0], $pkg;
   $i;
 }
@@ -121,7 +123,7 @@ sub looks_like_number {
   local $_ = shift;
 
   # checks from perlfaq4
-  return 1 unless defined;
+  return $] < 5.008005 unless defined;
   return 1 if (/^[+-]?\d+$/); # is a +/- integer
   return 1 if (/^([+-]?)(?=\d|\.\d)\d*(\.\d*)?([Ee]([+-]?\d+))?$/); # a C float
   return 1 if ($] >= 5.008 and /^(Inf(inity)?|NaN)$/i) or ($] >= 5.006001 and /^Inf$/i);
@@ -141,7 +143,7 @@ Scalar::Util - A selection of general-utility scalar subroutines
 
 =head1 SYNOPSIS
 
-    use Scalar::Util qw(blessed dualvar isweak readonly refaddr reftype tainted weaken isvstring looks_like_number);
+    use Scalar::Util qw(blessed dualvar isweak readonly refaddr reftype tainted weaken isvstring looks_like_number set_prototype);
 
 =head1 DESCRIPTION
 
@@ -243,6 +245,13 @@ is returned. Otherwise C<undef> is returned.
     $obj  = bless {}, "Foo";
     $type = reftype $obj;               # HASH
 
+=item set_prototype CODEREF, PROTOTYPE
+
+Sets the prototype of the given function, or deletes it if PROTOTYPE is
+undef. Returns the CODEREF.
+
+    set_prototype \&foo, '$$';
+
 =item tainted EXPR
 
 Return true if the result of EXPR is tainted
@@ -266,6 +275,25 @@ prevent the object being DESTROY-ed at its usual time.
     }
     # $ref is now undef
 
+Note that if you take a copy of a scalar with a weakened reference,
+the copy will be a strong reference.
+
+    my $var;
+    my $foo = \$var;
+    weaken($foo);                       # Make $foo a weak reference
+    my $bar = $foo;                     # $bar is now a strong reference
+
+This may be less obvious in other situations, such as C<grep()>, for instance
+when grepping through a list of weakened references to objects that may have
+been destroyed already:
+
+    @object = grep { defined } @object;
+
+This will indeed remove all references to destroyed objects, but the remaining
+references to objects will be strong, causing the remaining objects to never
+be destroyed because there is now always a strong reference to them in the
+@object array.
+
 =back
 
 =head1 KNOWN BUGS
@@ -275,7 +303,7 @@ show up as tests 8 and 9 of dualvar.t failing
 
 =head1 COPYRIGHT
 
-Copyright (c) 1997-2001 Graham Barr <gbarr@pobox.com>. All rights reserved.
+Copyright (c) 1997-2005 Graham Barr <gbarr@pobox.com>. All rights reserved.
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
 
